@@ -5,9 +5,11 @@ DoIP Server implementation for automotive diagnostics.
 This module provides the main DoIP (Diagnostics over IP) server functionality
 for handling automotive diagnostic communication protocols.
 """
+import logging
 import socket
 import struct
-import logging
+import time
+
 from .hierarchical_config_manager import HierarchicalConfigManager
 
 # DoIP Protocol constants
@@ -343,14 +345,27 @@ class DoIPServer:
                 # Handle both single response and list of responses
                 if responses:
                     if isinstance(responses, list):
-                        # Send multiple responses
+                        # Send multiple responses with delay support
                         for i, response in enumerate(responses):
+                            # Check if this response has a delay configuration
+                            delay_ms = self._get_response_delay(data, i)
+                            if delay_ms > 0:
+                                self.logger.info(
+                                    f"Delaying response {i+1} by {delay_ms}ms"
+                                )
+                                time.sleep(delay_ms / 1000.0)  # Convert ms to seconds
+
                             client_socket.send(response)
                             print(
                                 f"Sent response {i+1}/{len(responses)}: {response.hex()}"
                             )
                     else:
                         # Single response (backward compatibility)
+                        delay_ms = self._get_response_delay(data, 0)
+                        if delay_ms > 0:
+                            self.logger.info(f"Delaying response by {delay_ms}ms")
+                            time.sleep(delay_ms / 1000.0)  # Convert ms to seconds
+
                         client_socket.send(responses)
                         print(f"Sent response: {responses.hex()}")
         except Exception as e:
@@ -358,6 +373,71 @@ class DoIPServer:
         finally:
             client_socket.close()
 
+    def _get_response_delay(self, data, response_index):
+        """Get delay configuration for a specific response.
+
+        Args:
+            data (bytes): Original DoIP message data
+            response_index (int): Index of the response (0 for single response)
+
+        Returns:
+            int: Delay in milliseconds, 0 if no delay configured
+        """
+        try:
+            # Parse DoIP header to get payload type
+            if len(data) < 8:
+                return 0
+
+            payload_type = struct.unpack(">H", data[2:4])[0]
+
+            # Only apply delays to diagnostic messages (UDS)
+            if payload_type != PAYLOAD_TYPE_DIAGNOSTIC_MESSAGE:
+                return 0
+
+            # Extract UDS payload for service lookup
+            if len(data) < 12:  # DoIP header (8) + source (2) + target (2)
+                return 0
+
+            uds_payload = data[8:]  # Skip DoIP header
+            if len(uds_payload) < 4:
+                return 0
+
+            # Extract target address and UDS payload
+            target_address = struct.unpack(">H", uds_payload[2:4])[0]
+            uds_data = uds_payload[4:]
+
+            if not uds_data:
+                return 0
+
+            # Convert UDS payload to hex string for service matching
+            uds_hex = uds_data.hex().upper()
+
+            # Get service configuration
+            service_config = self.config_manager.get_uds_service_by_request(
+                uds_hex, target_address
+            )
+
+            if not service_config:
+                return 0
+
+            # Check for response-level delay first (higher priority)
+            responses = service_config.get("responses", [])
+            if response_index < len(responses):
+                response = responses[response_index]
+                if isinstance(response, dict):
+                    # Response-level delay overrides service-level delay
+                    return response.get("delay_ms", 0)
+                elif isinstance(response, str):
+                    # For string responses, fall back to service-level delay
+                    return service_config.get("delay_ms", 0)
+
+            # Fall back to service-level delay if no response-level delay
+            return service_config.get("delay_ms", 0)
+
+        except Exception as e:
+            self.logger.warning(f"Error getting response delay: {e}")
+            return 0
+
     def process_doip_message(self, data):
         """Process incoming DoIP message and return appropriate response(s).
 
@@ -464,7 +544,7 @@ class DoIPServer:
         return self.create_routing_activation_response(
             ROUTING_ACTIVATION_RESPONSE_CODE_SUCCESS,
             client_logical_address,
-            logical_address,
+            self._get_gateway_logical_address(),  # Use gateway logical address as source
         )
 
     def handle_diagnostic_message(self, payload):
@@ -784,6 +864,14 @@ class DoIPServer:
             )  # Service not supported in active session
 
         if service_config:
+            # Check if this service is configured for no response
+            no_response = service_config.get("no_response", False)
+            if no_response:
+                self.logger.info(
+                    f"Service {service_config.get('name', 'Unknown')} configured for no response"
+                )
+                return None  # Return None to indicate no response should be sent
+
             # Get responses for this service
             responses = service_config.get("responses", [])
             if responses:
@@ -799,10 +887,17 @@ class DoIPServer:
                 # Select response based on current index
                 response_template = responses[current_index]
 
-                # Process response template with request mirroring
-                response_hex = self.config_manager.process_response_with_mirroring(
-                    response_template, uds_hex
-                )
+                # Handle both string and dictionary response formats
+                if isinstance(response_template, dict):
+                    # New format: {"response": "0x...", "delay_ms": 100}
+                    response_hex = self.config_manager.process_response_with_mirroring(
+                        response_template.get("response", ""), uds_hex
+                    )
+                else:
+                    # Legacy format: "0x..." string
+                    response_hex = self.config_manager.process_response_with_mirroring(
+                        response_template, uds_hex
+                    )
 
                 # Update index for next time (cycle back to 0 when reaching end)
                 next_index = (current_index + 1) % len(responses)
@@ -925,10 +1020,18 @@ class DoIPServer:
     def create_routing_activation_response(
         self, response_code, client_logical_address, logical_address
     ):
-        """Create routing activation response"""
+        """Create routing activation response
+
+        Args:
+            response_code: DoIP routing activation response code
+            client_logical_address: Client's logical address (target in response)
+            logical_address: Gateway's logical address (source in response)
+        """
         # Create payload according to DoIP standard: !HHBLL format
         payload = struct.pack(">H", client_logical_address)  # Client logical address
-        payload += struct.pack(">H", logical_address)  # Logical address
+        payload += struct.pack(
+            ">H", logical_address
+        )  # Gateway logical address (source)
         payload += struct.pack(">B", response_code)  # Response code
         payload += struct.pack(">I", 0x00000000)  # Reserved (4 bytes)
         payload += struct.pack(">I", 0x00000000)  # VM specific (4 bytes)
@@ -1167,36 +1270,74 @@ class DoIPServer:
             self.logger.info(f"UDP Payload Type: 0x{payload_type:04X}")
             self.logger.info(f"UDP Payload Length: {payload_length}")
 
-            # Validate protocol version
-            if (
-                protocol_version != self.protocol_version
-                or inverse_protocol_version != self.inverse_protocol_version
-            ):
-                self.logger.warning(
-                    f"Invalid UDP protocol version: 0x{protocol_version:02X}"
-                )
-                return
-
             # Handle vehicle identification request
             if payload_type == PAYLOAD_TYPE_VEHICLE_IDENTIFICATION_REQUEST:
+                # For vehicle identification requests, accept both:
+                # - 0xFF/0x00 (ISO 13400-2:2019 default for vehicle identification)
+                # - 0x02/0xFD (standard DoIP protocol version)
+                is_valid_vehicle_id_version = (
+                    protocol_version == 0xFF and inverse_protocol_version == 0x00
+                ) or (
+                    protocol_version == self.protocol_version
+                    and inverse_protocol_version == self.inverse_protocol_version
+                )
+
+                if not is_valid_vehicle_id_version:
+                    self.logger.warning(
+                        f"Invalid protocol version for vehicle identification: "
+                        f"0x{protocol_version:02X}/0x{inverse_protocol_version:02X}"
+                    )
+                    return
                 self.logger.info("Processing vehicle identification request")
                 response = self.create_vehicle_identification_response()
                 if response:
                     self.udp_socket.sendto(response, addr)
                     self.logger.info(f"Sent vehicle identification response to {addr}")
             elif payload_type == PAYLOAD_TYPE_ENTITY_STATUS_REQUEST:
+                # Validate protocol version for entity status requests
+                if (
+                    protocol_version != self.protocol_version
+                    or inverse_protocol_version != self.inverse_protocol_version
+                ):
+                    self.logger.warning(
+                        f"Invalid UDP protocol version for entity status: "
+                        f"0x{protocol_version:02X}"
+                    )
+                    return
+
                 self.logger.info("Processing DoIP entity status request")
                 response = self.create_entity_status_response()
                 if response:
                     self.udp_socket.sendto(response, addr)
                     self.logger.info(f"Sent entity status response to {addr}")
             elif payload_type == PAYLOAD_TYPE_POWER_MODE_INFORMATION_REQUEST:
+                # Validate protocol version for power mode information requests
+                if (
+                    protocol_version != self.protocol_version
+                    or inverse_protocol_version != self.inverse_protocol_version
+                ):
+                    self.logger.warning(
+                        f"Invalid UDP protocol version for power mode: "
+                        f"0x{protocol_version:02X}"
+                    )
+                    return
+
                 self.logger.info("Processing power mode information request")
                 response = self.create_power_mode_response()
                 if response:
                     self.udp_socket.sendto(response, addr)
                     self.logger.info(f"Sent power mode information response to {addr}")
             else:
+                # Validate protocol version for other payload types
+                if (
+                    protocol_version != self.protocol_version
+                    or inverse_protocol_version != self.inverse_protocol_version
+                ):
+                    self.logger.warning(
+                        f"Invalid UDP protocol version: 0x{protocol_version:02X}"
+                    )
+                    return
+
                 self.logger.warning(
                     f"Unsupported UDP payload type: 0x{payload_type:04X}"
                 )
