@@ -6,10 +6,10 @@ Handles loading and parsing of multiple YAML configuration files with ECU hierar
 
 import logging
 import os
+import re
 from typing import Any, Dict, List, Optional
 
 import yaml
-import re
 
 
 class HierarchicalConfigManager:
@@ -304,6 +304,14 @@ gateway:
             os.path.join(
                 "config", "ecus", "transmission", service_file
             ),  # Transmission services
+            os.path.join("config", "ecus", "esp", service_file),  # ESP services
+            os.path.join(
+                "config", "ecus", "steering", service_file
+            ),  # Steering services
+            os.path.join("config", "ecus", "bcm", service_file),  # BCM services
+            os.path.join("config", "ecus", "gateway", service_file),  # Gateway services
+            os.path.join("config", "ecus", "hvac", service_file),  # HVAC services
+            os.path.join("config", "ecus", "airbag", service_file),  # Airbag services
         ]
 
         for path in possible_paths:
@@ -789,6 +797,34 @@ gateway:
         # Validate UDS services
         if not self.uds_services:
             self.logger.warning("No UDS services loaded")
+        else:
+            # Validate service configurations
+            for service_name, service_config in self.uds_services.items():
+                # Validate no_response configuration
+                if "no_response" in service_config:
+                    no_response = service_config.get("no_response")
+                    if not isinstance(no_response, bool):
+                        self.logger.error(
+                            f"Service '{service_name}': no_response must be a boolean value"
+                        )
+                        return False
+
+                # If no_response is True, responses should be empty or not present
+                no_response = service_config.get("no_response", False)
+                if no_response:
+                    responses = service_config.get("responses", [])
+                    if responses:
+                        self.logger.warning(
+                            f"Service '{service_name}': no_response is True but responses are configured. "
+                            "Responses will be ignored."
+                        )
+                else:
+                    # If no_response is False or not set, validate that responses exist
+                    responses = service_config.get("responses", [])
+                    if not responses:
+                        self.logger.warning(
+                            f"Service '{service_name}': no responses configured and no_response is not set to True"
+                        )
 
         self.logger.info("Configuration validation passed")
         return True
